package App::Gitc::Test;
{
  $App::Gitc::Test::VERSION = '0.58';
}
use strict;
use warnings;

#    Copyright 2012 Grant Street Group, All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

use Test::More '';
use App::Gitc::Util qw( branch_point unpromoted );
use Exporter 'import';

BEGIN {
    our @EXPORT = qw( branch_point_is unpromoted_is );
};

sub branch_point_is {
    my ( $ref, $expected, $message ) = @_;
    my $sha1 = branch_point($ref);
    Test::More::is( $sha1, $expected, $message );
}

sub unpromoted_is {
    my ( $source, $target, $expected, $message ) = @_;
    my @changesets = unpromoted( $source, $target );
    Test::More::is_deeply(\@changesets, $expected, $message);
}

1;

__END__

=pod

=head1 NAME

App::Gitc::Test

=head1 VERSION

version 0.58

=head1 AUTHOR

Grant Street Group

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2013 by Grant Street Group.

This is free software, licensed under:

  The GNU Affero General Public License, Version 3, November 2007

=cut
